# == Schema Information
# Schema version: 20090612184651
#
# Table name: games
#
#  id                  :integer         not null, primary key
#  date                :datetime        not null
#  maximum_memberships :integer         not null
#  created_at          :datetime
#  updated_at          :datetime
#  championship_id     :integer
#  game_manager_id     :integer
#  state               :string(255)     default("open")
#  winner              :string(255)     default("none"), not null
#

class Game < ActiveRecord::Base
  
  hobo_model # Don't put anything above this
  
  fields do
    date :datetime, :null => false
    maximum_memberships :integer, :null => false
    state enum_string(:open, :closed), :default => "open"
    winner :string, :null => false, :default => 'none'
    timestamps
  end
  
  belongs_to :championship 
  belongs_to :game_manager, :class_name => "User", :creator => true
  has_many :memberships, :class_name => "GameMembership", :dependent => :destroy
  has_many :members, :through => :memberships, :source => :user
  has_many :comments, :dependent => :destroy

  set_default_order "created_at desc"

  named_scope :interesting_games, lambda { |user| {
      :select => "games.*",
      :joins => ", championships, championship_memberships", 
      :conditions=> ["championship_memberships.user_id =  ? AND championship_memberships.championship_id = championships.id and games.championship_id = championships.id  AND championships.state = 'open'", user],
      :order => "games.created_at DESC"
    }}
  
  validates_presence_of :championship
  validates_presence_of :date
  validates_presence_of :maximum_memberships
  validates_numericality_of :maximum_memberships, :only_integer => true, :greater_than_or_equal_to => 2, :less_than_or_equal_to => 30
  validates_presence_of :game_manager
  
  #TODO remove after transforming this into a lifecycle
  def after_create
    championship.members.each { |member|
      member.notify_game_creation(self)
    }
  end
  
  
  #calculates the winner taking int account the game_memberships
  def compute_winner!
    self.winner = 'none'
    biggest_score = -100000000
    memberships.group_by(&:team).each do |team, team_memberships| 
      next unless team #team may be nil
      team_score = team_memberships.inject(0){|sum,membership| sum + membership.score}
      team_name = team_memberships[0].team
      if team_score > biggest_score
        biggest_score = team_score
        self.winner = team_name
      elsif team_score == biggest_score
        self.winner = 'none'
      end
    end
    save!
  end
  
  
  def randomize_teams
    return unless championship.state.is_open? and state.is_open?
    team = 'B'
    
    memberships.pop_all_randomly{|game_membership, index| 
      team = team == 'A' ? 'B' : 'A'
      #don't know why, but game_meberships is an array and index is nil
      game_membership[0].team = team
      game_membership[0].save      
    }
  end
  
  def name
    #"Game @ #{date}""
    t = teams
    if (t.size == 0 && state.is_open?)
      "New Game"
    elsif (t.size == 0 && state.is_closed?)
      "Closed Game"
    elsif (t.size == 1)
      "#{t.first} vs World"
    else
      t.join(" vs ")
    end
  end
  
  def teams
    memberships.group_by(&:team).keys.select{ |x| x and x.strip != ""}
  end
  
  # --- Hobo Permissions --- #
  
  def create_permitted?
    return false unless acting_user.signed_up?
    return false unless  !winner_changed?
    return false unless game_manager_is?(acting_user)
     (acting_user.administrator? || championship && championship.state.is_open? && championship.championship_manager == acting_user) && game_manager == acting_user
  end
  
  def update_permitted?
    return false unless !game_manager_changed? && !winner_changed? && !championship_changed?
     (acting_user.administrator? || championship.state.is_open? && (championship.championship_manager == acting_user || game_manager == acting_user))
  end
  
  def destroy_permitted?
   (acting_user.administrator? || championship.state.is_open? && championship.championship_manager == acting_user)
  end
  
  def view_permitted?(attribute)
    true
  end
  
  def randomize_teams_permitted?
    return false unless acting_user.signed_up?
    return false unless championship.state.is_open? and state.is_open?
    return false unless acting_user.administrator?  or championship.championship_manager_is?(acting_user) or game_manager_is?(acting_user)
    return !members.empty?
  end
  
  
  
end
